@@ -6,48 +6,46 @@ WWW::PDB - Perl interface to the Protein Data Bank
 
 =head1 SYNOPSIS
 
-  use WWW::PDB;
-  my $pdb = new WWW::PDB;
+  use WWW::PDB qw(:all);
+
+  # set directory for caching downloads
+  WWW::PDB->cache('/foo/bar');
   
-  my $fh = $pdb->get_structure('2ili');
+  my $fh = get_structure('2ili');
   print while <$fh>;
   
-  for($pdb->keyword_query('carbonic anhydrase')) {
-      printf(
-          "%s\t%s\t[%s]\n",
-          $_,
-          $pdb->get_primary_citation_title($_),
-          join(', ', $pdb->get_chains($_))
-      );
+  my @pdbids = WWW::PDB->keyword_query('carbonic anhydrase');
+  for(@pdbids) {
+      my $citation = WWW::PDB->get_primary_citation_title($_),
+      my @chains   = WWW::PDB->get_chains($_);
+      printf("%s\t%s\t[%s]\n", $_, $citation, join(', ', @chains));
   }
 
   my $seq = q(
       VLSPADKTNVKAAWGKVGAHAGEYGAEALERMFLSFPTTK
       TYFPHFDLSHGSAQVKGHGKKVADALTAVAHVDDMPNAL
   );
-  print $pdb->blast($seq, 10.0, 'BLOSUM62', 'HTML');
-
-=head1 INTRODUCTION
-
-The Protein Data Bank (PDB) was established in 1971 as a repository of the
-atomic coordinates of protein structures (Bernstein I<et al.>, 1997). It
-has since outgrown that role, proving invaluable not only to the research
-community but also to students and educators (Berman I<et al.>, 2002).
+  print WWW::PDB->blast($seq, 10.0, 'BLOSUM62', 'HTML');
 
 =head1 DESCRIPTION
 
-This module is an object-oriented interface to the Protein Data Bank. It
-provides methods for retrieving files, optionally caching them
-locally. Additionally, it wraps the functionality of the PDB's SOAP web
-services.
+The Protein Data Bank (PDB) was established in 1971 as a repository of the
+atomic coordinates of protein structures (Bernstein I<et al.>, 1997).  It
+has since outgrown that role, proving invaluable not only to the research
+community but also to students and educators (Berman I<et al.>, 2002).
+
+L<WWW::PDB> is a Perl interface to the Protein Data Bank.  It provides
+functions for retrieving files, optionally caching them locally.
+Additionally, it wraps the functionality of the PDB's SOAP web services.
 
 =cut
 
-use 5.008;
+use 5.006;
 use strict;
 use warnings;
 
 use Carp;
+use Exporter;
 use Fcntl;
 use File::Path;
 use File::Spec;
@@ -56,115 +54,248 @@ use IO::Uncompress::Gunzip;
 use Net::FTP;
 use SOAP::Lite;
 
-our @ISA = qw();
+use constant {
+    BOOLEAN => 0,
+    DOUBLE  => 1,
+    INT     => 2,
+    SELF    => 3,
+    STRING  => 4,
+};
 
-our $VERSION = '0.00_01';
+our @ISA = qw(Exporter);
+our $VERSION = '0.00_02';
 $VERSION = eval $VERSION;
 
-=head1 CONSTRUCTOR
+our %EXPORT_TAGS = (
+    file   => [qw(get_structure get_structure_factors)],
+    status => [qw(get_status is_current is_obsolete is_unreleased
+                  is_model is_unknown)],
+);
+our @EXPORT_OK = map {@$_} values %EXPORT_TAGS;
+$EXPORT_TAGS{all} = \@EXPORT_OK;
+
+my($uri, $proxy, $ftp, $cache, $soap);
+
+=head1 FUNCTIONS
+
+=head2 CUSTOMIZATION
+
+Let's start with some functions that let you customize how the module does
+its job.  You probably won't play with any of these very often (if at all)
+except for C<cache>, which is recommended for anyone that expects to do
+extensive work with a set of files: that way you don't waste resources
+downloading them each time.
 
 =over 4
 
-=item new ( [ OPTIONS ] )
+=item WWW::PDB->ftp( [ $HOST ] )
 
-Prepares a new L<WWW::PDB> object with the specified options, which are
-passed in as key-value pairs, as in a hash. Accepted options are:
-
-I<uri> - URI for the PDB web services. Defaults to
-E<lt>http://www.pdb.org/pdb/services/pdbwsE<gt>.
-
-I<proxy> - Proxy for the PDB web services. Defaults to
-E<lt>http://www.pdb.org/pdb/services/pdbwsE<gt>.
-
-I<ftp> - Host name for the PDB FTP archive. Defaults to F<ftp.wwpdb.org>.
-
-I<cache> - Local cache directory. If defined, the object will look for
-files here first and also use this directory to store any downloads.
-
-Options not listed above are ignored, and, appropriately, all options
-are optional.
+Returns the host name for the PDB FTP archive, first setting it to $FTP if
+it's specified.  Default value is F<ftp.wwpdb.org>.
 
 =cut
 
-sub new {
-    my($class, %opts) = @_;
-    $opts{uri}   ||= 'http://www.pdb.org/pdb/services/pdbws';
-    $opts{proxy} ||= 'http://www.pdb.org/pdb/services/pdbws';
-    $opts{ftp}   ||= 'ftp.wwpdb.org';
-    $opts{cache} ||= undef;
-    return bless({
-        service => SOAP::Lite->uri($opts{uri})->proxy($opts{proxy}),
-        ftp     => $opts{ftp},
-        cache   => $opts{cache},
-    }, $class);
+sub ftp {
+    return $ftp = $_[1] ? $_[1] : $ftp || 'ftp.wwpdb.org';
+}
+
+=item WWW::PDB->cache( [ $DIR ] )
+
+Returns the local cache directory, first setting it to $DIR if it's
+specified.  If C<defined>, the module will look for files here first and
+also use the directory to store any downloads.
+
+=cut
+
+sub cache {
+    $cache = $_[1] if $_[1];
+    return $cache;
+}
+
+=item WWW::PDB->ns( [ $URI ] )
+
+Returns the namespace URI for the PDB web services, first setting it to $URI
+if it's specified.  Default value is http://www.pdb.org/pdb/services/pdbws.
+
+=cut
+
+sub ns {
+    my $tmp = $uri;
+    $uri = $_[1] ? $_[1] : $uri || 'http://www.pdb.org/pdb/services/pdbws';
+    $_[0]->soap->ns($uri) unless $tmp && $tmp eq $uri;
+    return $uri;
+}
+
+=item WWW::PDB->proxy( [ $URI ] )
+
+Returns the proxy for the PDB web services, first setting it to $URI if it's
+specified.  Default value is http://www.pdb.org/pdb/services/pdbws.
+
+=cut
+
+sub proxy {
+    my $tmp = $proxy;
+    $proxy = $_[1] ? $_[1] : $proxy || 'http://www.pdb.org/pdb/services/pdbws';
+    $_[0]->soap->proxy($proxy) unless $tmp && $tmp eq $proxy;
+    return $proxy;
+}
+
+=item WWW::PDB->soap( [ $CLIENT ] )
+
+Returns the client L<SOAP::Lite> object used by this module to talk to
+the PDB's SOAP interface, first setting it to $CLIENT if it's specified.
+It's best not to access it directly, but if you must, this is how.
+
+=cut
+
+sub soap {
+    return $soap = $_[1] ? $_[1] : $soap ||
+        SOAP::Lite->ns($_[0]->ns)->proxy($_[0]->proxy);
 }
 
 =back
 
-=head1 METHODS
-
-This module is object-oriented, so all methods should be called on a
-L<WWW::PDB> instance.
-
 =head2 FILE RETRIEVAL
 
-Each of the following methods takes a PDB ID as input and returns a file
-handle (or C<undef> on failure).
+Each of the following functions takes a PDB ID as input and returns a file
+handle (or C<undef> on failure).  You can import these into your namespace
+with the C<file> tag, as in C<use WWW::PDB qw(:file)>.
 
 =over 4
 
-=item get_structure ( PDBID )
+=item get_structure( $PDBID )
 
 Retrieves the structure in PDB format.
 
 =cut
 
 sub get_structure {
-    my $self  = shift;
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
+    my $class = shift;
     my $pdbid = lc(shift);
     return $pdbid =~ /^.(..).$/
-      ? $self->_get_file(
-            qw(pub pdb data structures divided pdb),
-            $1, "pdb${pdbid}.ent.gz"
-        )
-      : undef;
+        ? $class->_get_file(qw(pub pdb data structures divided pdb), $1,
+        "pdb${pdbid}.ent.gz") : undef;
 }
 
-=item get_structure_factors ( PDBID )
+=item get_structure_factors( $PDBID )
 
 Retrieves the structure factors file.
 
 =cut
 
 sub get_structure_factors {
-    my $self  = shift;
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
+    my $class = shift;
     my $pdbid = lc(shift);
     return $pdbid =~ /^.(..).$/
-      ? $self->_get_file(
-            qw(pub pdb data structures divided structure_factors),
-            $1, "r${pdbid}sf.ent.gz"
-        )
-      : undef;
+        ? $class->_get_file(qw(pub pdb data structures divided
+        structure_factors), $1, "r${pdbid}sf.ent.gz") : undef;
 }
 
 =back
 
-=head2 UTILITY
+=head2 PDB ID STATUS
 
-This section is dedicated to utility methods.
+The following functions deal with the status of PDB IDs.  You can import
+them into your namespace with the C<status> tag:
+C<use WWW::PDB qw(:status)>.
 
 =over 4
 
-=item service
+=item get_status( $PDBID )
 
-Hopefully you don't need to play directly with the backing L<SOAP::Lite>
-object, but if you do, this is how.
+Finds the status of the structure with the given $PDBID.  Return is in
+C<qw(CURRENT OBSOLETE UNRELEASED MODEL UNKNOWN)>.
 
 =cut
 
-sub service {
-    my $self = shift;
-    return $self->{service};
+sub get_status {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
+    return 'UNKNOWN' if length($_[1]) != 4;
+    my $class = shift;
+    my $pdbid = _to_string(shift);
+    my $ret   = $self->_call(
+        'getIdStatus', $pdbid
+    );
+    return $ret;
+}
+
+=item is_current( $PDBID )
+
+Checks whether or not the specified $PDBID corresponds to a current
+structure.  Implemented for orthogonality, all this does is check
+if C<get_status> returns C<CURRENT>.
+
+=cut
+
+sub is_current {
+    my $class = UNIVERSAL::isa($_[0], __PACKAGE__) ? shift : __PACKAGE__;
+    return $class->get_status(@_) eq 'CURRENT';
+}
+
+=item is_obsolete( $PDBID )
+
+Checks whether or not the specified $PDBID corresponds to an obsolete
+structure.  This is actually defined by the PDB web services interface.
+
+=cut
+
+sub is_obsolete {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
+    my $self  = shift;
+    my $pdbid = _to_string(shift);
+    my $ret   = $self->_call(
+        'isStructureIdObsolete', $pdbid
+    );
+    return $ret;
+}
+
+=item is_unreleased( $PDBID )
+
+Checks whether or not the specified $PDBID corresponds to an unreleased
+structure.  Implemented for orthogonality, all this does is check
+if C<get_status> returns C<UNRELEASED>.
+
+=cut
+
+sub is_unreleased {
+    my $class = UNIVERSAL::isa($_[0], __PACKAGE__) ? shift : __PACKAGE__;
+    return $class->get_status(@_) eq 'UNRELEASED';
+}
+
+=item is_model( $PDBID )
+
+Checks whether or not the specified $PDBID corresponds to a model
+structure.  Implemented for orthogonality, all this does is check
+if C<get_status> returns C<MODEL>.
+
+=cut
+
+sub is_model {
+    my $class = UNIVERSAL::isa($_[0], __PACKAGE__) ? shift : __PACKAGE__;
+    return $class->get_status(@_) eq 'MODEL';
+}
+
+=item is_unknown( $PDBID )
+
+Checks whether or not the specified $PDBID is unknown.  Implemented
+for orthogonality, all this does is check if C<get_status> returns
+C<UNKNOWN>.
+
+=cut
+
+sub is_unknown {
+    my $class = UNIVERSAL::isa($_[0], __PACKAGE__) ? shift : __PACKAGE__;
+    return $class->get_status(@_) eq 'UNKNOWN';
 }
 
 =back
@@ -175,13 +306,13 @@ The following methods are the interface to the PDB web services.
 
 =over 4
 
-=item blast ( SEQUENCE , CUTOFF , MATRIX , OUTPUT_FORMAT )
+=item blast( $SEQUENCE , $CUTOFF , $MATRIX , $OUTPUT_FORMAT )
 
-=item blast ( PDBID , CHAINID, CUTOFF , MATRIX , OUTPUT_FORMAT )
+=item blast( $PDBID , $CHAINID, $CUTOFF , $MATRIX , $OUTPUT_FORMAT )
 
-=item blast ( SEQUENCE , CUTOFF )
+=item blast( $SEQUENCE , $CUTOFF )
 
-=item blast ( PDBID , CHAINID , CUTOFF )
+=item blast( $PDBID , $CHAINID , $CUTOFF )
 
 Performs a BLAST against sequences in the PDB and returns the output of
 the BLAST program. XML is used if the output format is unspecified.
@@ -189,39 +320,33 @@ the BLAST program. XML is used if the output format is unspecified.
 =cut
 
 sub _blast_pdb {
-    my $self          = shift;
-    my $sequence      = _to_string(shift);
-    my $cutoff        = _to_double(shift);
-    my $matrix        = _to_string(shift);
-    my $output_format = _to_string(shift);
-    my $ret           = $self->_call(
+    my($self, $sequence, $cutoff, $matrix, $output_format) = 
+        _wrap(\@_ => [SELF, STRING, DOUBLE, STRING, STRING]);
+
+    my $ret = $self->_call(
         'blastPDB', $sequence, $cutoff, $matrix, $output_format
     );
     return $ret;
 }
 
 sub _blast_structure_id_pdb {
-    # I keep getting "ERROR: No Results Found" using the PDB's 5 argument
-    # form of blastPDB. Here's a workaround:
     my $self = shift;
-    my $seq  = $self->get_sequence(shift, shift);
+
+    # I keep getting "ERROR: No Results Found" using the PDB's 5 argument
+    # form of blastPDB. Here's a workaround:    
+    my $seq = $self->get_sequence(shift, shift);
     return $self->blast($seq, @_);
 
-#   my $pdbid         = _to_string(shift);
-#   my $chainid       = _to_string(shift);
-#   my $cutoff        = _to_double(shift);
-#   my $matrix        = _to_string(shift);
-#   my $output_format = _to_string(shift);
-#   my $ret           = $self->_call(
+#   my($self, $pdbid, $chainid, $cutoff, $matrix, $output_format) =
+#       _wrap(\@_ => [SELF, STRING, STRING, DOUBLE, STRING, STRING]);
+#   my $ret = $self->_call(
 #       'blastPDB', $pdbid, $chainid, $cutoff, $matrix, $output_format
 #   );
 #   return $ret;
 }
 
 sub _blast_query_xml {
-    my $self     = shift;
-    my $sequence = _to_string(shift);
-    my $cutoff   = _to_double(shift);
+    my($self, $sequence, $cutoff) = _wrap(\@_ => [SELF, STRING, DOUBLE]);
     my $ret      = $self->_call(
         'blastQueryXml', $sequence, $cutoff
     );
@@ -229,10 +354,8 @@ sub _blast_query_xml {
 }
 
 sub _blast_structure_id_query_xml {
-    my $self    = shift;
-    my $pdbid   = _to_string(shift);
-    my $chainid = _to_string(shift);
-    my $cutoff  = _to_double(shift);
+    my($self, $pdbid, $chainid, $cutoff)
+        = _wrap(\@_ => [SELF, STRING, STRING, DOUBLE]);
     my $ret     = $self->_call(
         'blastStructureIdQueryXml', $pdbid, $chainid, $cutoff
     );
@@ -240,6 +363,9 @@ sub _blast_structure_id_query_xml {
 }
 
 sub blast {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self = shift;
     my $ret;
     my $c = scalar(@_);
@@ -247,13 +373,13 @@ sub blast {
     elsif($c == 5) { $ret = $self->_blast_structure_id_pdb(@_) }
     elsif($c == 2) { $ret = $self->_blast_query_xml(@_) }
     elsif($c == 3) { $ret = $self->_blast_structure_id_query_xml(@_) }
-    else { confess "Called blast with unexpected number of arguments" }
+    else { confess 'Called blast with unexpected number of arguments' }
     return $ret;
 }
 
-=item fasta ( SEQUENCE , CUTOFF )
+=item fasta( $SEQUENCE , $CUTOFF )
 
-=item fasta ( PDBID , CHAINID , CUTOFF )
+=item fasta( $PDBID , $CHAINID , $CUTOFF )
 
 Takes a sequence or PDB ID and chain identifier and runs FASTA using the
 specified cut-off. The results are overloaded to give PDB IDs when used
@@ -276,34 +402,38 @@ sub _fasta_query {
 }
 
 sub _fasta_structure_id_query {
-    my $self    = shift;
-    my $pdbid   = _to_string(shift);
-    my $chainid = _to_string(shift);
-    my $cutoff  = _to_double(shift);
-    my $ret     = $self->_call(
+    my($self, $pdbid, $chainid, $cutoff)
+        = _wrap(\@_ => [SELF, STRING, STRING, DOUBLE]);
+    my $ret = $self->_call(
         'fastaStructureIdQuery', $pdbid, $chainid, $cutoff
     );
     return $ret;
 }
 
 sub fasta {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self = shift;
     my $c    = scalar(@_);
     my $ret;
     if   ($c == 2) { $ret = $self->_fasta_query(@_) }
     elsif($c == 3) { $ret = $self->_fasta_structure_id_query(@_) }
-    else { confess "Called fasta with unexpected number of arguments" }
+    else { confess 'Called fasta with unexpected number of arguments' }
     $_ = bless(\"$_", 'WWW::PDB::_FastaResult') for @$ret;
     return wantarray ? @$ret : $ret;
 }
 
-=item get_chain_length ( PDBID , CHAINID )
+=item get_chain_length( $PDBID , $CHAINID )
 
 Returns the length of the specified chain.
 
 =cut
 
 sub get_chain_length {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self    = shift;
     my $pdbid   = _to_string(shift);
     my $chainid = _to_string(shift);
@@ -313,7 +443,7 @@ sub get_chain_length {
     return $ret;
 }
 
-=item get_chains ( PDBID )
+=item get_chains( $PDBID )
 
 Returns a list of all the chain identifiers for a given structure, or a
 reference to such a list in scalar context.
@@ -321,6 +451,9 @@ reference to such a list in scalar context.
 =cut
 
 sub get_chains {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self  = shift;
     my $pdbid = _to_string(shift);
     my $ret   = $self->_call(
@@ -329,7 +462,7 @@ sub get_chains {
     return wantarray ? @$ret : $ret;
 }
 
-=item get_cif_chain ( PDBID , CHAINID )
+=item get_cif_chain( $PDBID , $CHAINID )
 
 Converts the specified author-assigned chain identifier to its mmCIF
 equivalent.
@@ -337,6 +470,9 @@ equivalent.
 =cut
 
 sub get_cif_chain {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self    = shift;
     my $pdbid   = _to_string(shift);
     my $chainid = _to_string(shift);
@@ -346,7 +482,7 @@ sub get_cif_chain {
     return $ret;
 }
 
-=item get_cif_chain_length ( PDBID , CHAINID )
+=item get_cif_chain_length( $PDBID , $CHAINID )
 
 Returns the length of the specified chain, just like C<get_chain_length>,
 except it expects the chain identifier to be the mmCIF version.
@@ -354,6 +490,9 @@ except it expects the chain identifier to be the mmCIF version.
 =cut
 
 sub get_cif_chain_length {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self    = shift;
     my $pdbid   = _to_string(shift);
     my $chainid = _to_string(shift);
@@ -363,7 +502,7 @@ sub get_cif_chain_length {
     return $ret;
 }
 
-=item get_cif_chains ( PDBID )
+=item get_cif_chains( $PDBID )
 
 Returns a list of all the mmCIF chain identifiers for a given structure, or
 a reference to such a list in scalar context.
@@ -371,6 +510,9 @@ a reference to such a list in scalar context.
 =cut
 
 sub get_cif_chains {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self  = shift;
     my $pdbid = _to_string(shift);
     my $ret   = $self->_call(
@@ -379,7 +521,7 @@ sub get_cif_chains {
     return wantarray ? @$ret : $ret;
 }
 
-=item get_cif_residue ( PDBID , CHAINID , RESIDUEID )
+=item get_cif_residue( $PDBID , $CHAINID , $RESIDUEID )
 
 Converts the specified author-assigned residue identifier to its mmCIF
 equivalent.
@@ -387,6 +529,9 @@ equivalent.
 =cut
 
 sub get_cif_residue {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self      = shift;
     my $pdbid     = _to_string(shift);
     my $chainid   = _to_string(shift);
@@ -397,7 +542,7 @@ sub get_cif_residue {
     return $ret;
 }
 
-=item get_current_pdbids ( )
+=item get_current_pdbids( )
 
 Returns a list of the identifiers (PDB IDs) corresponding to "current"
 structures (i.e. not obsolete, models, etc.), or a reference to such a
@@ -406,6 +551,9 @@ list in scalar context.
 =cut
 
 sub get_current_pdbids {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self = shift;
     my $ret  = $self->_call(
         'getCurrentPdbIds'
@@ -413,9 +561,9 @@ sub get_current_pdbids {
     return wantarray ? @$ret : $ret;
 }
 
-=item get_ec_nums ( PDBIDS )
+=item get_ec_nums( @PDBIDS )
 
-=item get_ec_nums ( )
+=item get_ec_nums( )
 
 Retrieves the Enzyme Classification (EC) numbers associated with the
 specified PDB IDs or with all PDB structures if called with no arguments. 
@@ -423,6 +571,9 @@ specified PDB IDs or with all PDB structures if called with no arguments.
 =cut
 
 sub get_ec_nums {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self = shift;
     my $ret;
     if(@_) {
@@ -440,7 +591,7 @@ sub get_ec_nums {
     return wantarray ? @$ret : $ret;
 }
 
-=item get_entities ( PDBID )
+=item get_entities( $PDBID )
 
 Returns a list of the entity IDs for a given structure, or a reference
 to such a list in scalar context.
@@ -448,6 +599,9 @@ to such a list in scalar context.
 =cut
 
 sub get_entities {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self  = shift;
     my $pdbid = _to_string(shift);
     my $ret   = $self->_call(
@@ -456,13 +610,16 @@ sub get_entities {
     return $ret && wantarray ? @$ret : $ret;
 }
 
-=item get_genome_details ( )
+=item get_genome_details( )
 
 Retrieves genome details for all PDB structures.
 
 =cut
 
 sub get_genome_details {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self = shift;
     my $ret  = $self->_call(
         'getGenomeDetails'
@@ -470,13 +627,16 @@ sub get_genome_details {
     return $ret && wantarray ? @$ret : $ret;
 }
 
-=item get_kabsch_sander ( PDBID , CHAINID )
+=item get_kabsch_sander( $PDBID , $CHAINID )
 
 Finds secondary structure for the given chain.
 
 =cut
 
 sub get_kabsch_sander {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self    = shift;
     my $pdbid   = _to_string(shift);
     my $chainid = _to_string(shift);
@@ -486,7 +646,7 @@ sub get_kabsch_sander {
     return $ret;
 }
 
-=item get_obsolete_pdbids ( )
+=item get_obsolete_pdbids( )
 
 Returns a list of the identifiers (PDB IDs) corresponding to obsolete
 structures, or a reference to such a list in scalar context.
@@ -494,6 +654,9 @@ structures, or a reference to such a list in scalar context.
 =cut
 
 sub get_obsolete_pdbids {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self = shift;
     my $ret  = $self->_call(
         'getObsoletePdbIds'
@@ -501,7 +664,7 @@ sub get_obsolete_pdbids {
     return $ret && wantarray ? @$ret : $ret;
 }
 
-=item get_primary_citation_title ( PDBID )
+=item get_primary_citation_title( $PDBID )
 
 Finds the title of the specified structure's primary citation (if it has
 one).
@@ -509,6 +672,9 @@ one).
 =cut
 
 sub get_primary_citation_title {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self  = shift;
     my $pdbid = _to_string(shift);
     my $ret   = $self->_call(
@@ -517,13 +683,16 @@ sub get_primary_citation_title {
     return $ret;
 }
 
-=item get_pubmed_ids ( )
+=item get_pubmed_ids( )
 
 Retrieves the PubMed IDs associated with all PDB structures.
 
 =cut
 
 sub get_pubmed_ids {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self = shift;
     my $ret  = $self->_call(
         'getPubmedIdForAllStructures'
@@ -531,13 +700,16 @@ sub get_pubmed_ids {
     return $ret && wantarray ? @$ret : $ret;
 }
 
-=item get_pubmed_id ( PDBID )
+=item get_pubmed_id( $PDBID )
 
 Retrieves the PubMed ID associated with the specified structure.
 
 =cut
 
 sub get_pubmed_id {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self  = shift;
     my $pdbid = _to_string(shift);
     my $ret   = $self->_call(
@@ -546,13 +718,16 @@ sub get_pubmed_id {
     return $ret;
 }
 
-=item get_release_dates ( PDBIDS )
+=item get_release_dates( @PDBIDS )
 
 Maps the given PDB IDs to their release dates.
 
 =cut
 
 sub get_release_dates {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self   = shift;
     my @pdbids = map(_to_string($_), map { ref($_) ? @$_ : $_ } @_);
     my $ret    = $self->_call(
@@ -561,13 +736,16 @@ sub get_release_dates {
     return $ret && wantarray ? @$ret : $ret;
 }
 
-=item get_sequence ( PDBID , CHAINID )
+=item get_sequence( $PDBID , $CHAINID )
 
 Retrieves the sequence of the specified chain.
 
 =cut
 
 sub get_sequence {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self    = shift;
     my $pdbid   = _to_string(shift);
     my $chainid = _to_string(shift);
@@ -577,14 +755,17 @@ sub get_sequence {
     return $ret;
 }
 
-=item get_space_group ( PDBID )
+=item get_space_group( $PDBID )
 
 Returns the space group of the specified structure (the
-symmetry.space_group_name_H_M field according to the mmCIF dictionary).
+C<symmetry.space_group_name_H_M> field according to the mmCIF dictionary).
 
 =cut
 
 sub get_space_group {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self  = shift;
     my $pdbid = _to_string(shift);
     my $ret   = $self->_call(
@@ -593,13 +774,16 @@ sub get_space_group {
     return $ret;
 }
 
-=item homology_reduction_query ( PDBIDS , CUTOFF )
+=item homology_reduction_query( @PDBIDS , $CUTOFF )
 
 Reduces the set of PDB IDs given as input based on sequence homology.
 
 =cut
 
 sub homology_reduction_query {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self   = shift;
     my $cutoff = _to_int(int(pop));
     my @pdbids = map(_to_string($_), map { ref($_) ? @$_ : $_ } @_);
@@ -609,7 +793,7 @@ sub homology_reduction_query {
     return $ret && wantarray ? @$ret : $ret;
 }
 
-=item keyword_query ( KEYWORD_EXPR [, EXACT_MATCH , [ AUTHORS_ONLY ] ] )
+=item keyword_query( $KEYWORD_EXPR [, $EXACT_MATCH [, $AUTHORS_ONLY ] ] )
 
 Runs a keyword query with the specified expression. Search can be made
 stricter by requiring an exact match or restricting the search to
@@ -619,6 +803,9 @@ a list of PDB IDs or a reference to such a list in scalar context.
 =cut
 
 sub keyword_query {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self         = shift;
     my $keyword      = _to_string(shift);
     my $exact_match  = _to_boolean(shift || 0);
@@ -629,7 +816,7 @@ sub keyword_query {
     return $ret && wantarray ? @$ret : $ret;
 }
 
-=item pubmed_abstract_query ( KEYWORD_EXPR )
+=item pubmed_abstract_query( $KEYWORD_EXPR )
 
 Runs a keyword query on PubMed Abstracts. Returns a list of PDB IDs or
 a reference to such a list in scalar context.
@@ -637,6 +824,9 @@ a reference to such a list in scalar context.
 =cut
 
 sub pubmed_abstract_query {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self    = shift;
     my $keyword = _to_string(shift);
     my $ret     = $self->_call(
@@ -647,99 +837,6 @@ sub pubmed_abstract_query {
 
 =back
 
-=head3 PDB ID STATUS METHODS
-
-The following methods deal with the status of PDB IDs.
-
-=over 4
-
-=item get_status ( PDBID )
-
-Finds the status of the structure with the given PDB ID. Return is one
-of C<qw(CURRENT OBSOLETE UNRELEASED MODEL UNKNOWN)>.
-
-=cut
-
-sub get_status {
-    return 'UNKNOWN' if length($_[1]) != 4;
-    my $self  = shift;
-    my $pdbid = _to_string(shift);
-    my $ret   = $self->_call(
-        'getIdStatus', $pdbid
-    );
-    return $ret;
-}
-
-=item is_current ( PDBID )
-
-Checks whether or not the specified PDB ID corresponds to a current
-structure. Implemented for orthogonality, all this does is check
-if C<get_status> returns C<CURRENT>.
-
-=cut
-
-sub is_current {
-    my $self = shift;
-    return $self->get_status(@_) eq 'CURRENT';
-}
-
-=item is_obsolete ( PDBID )
-
-Checks whether or not the specified PDB ID corresponds to an obsolete
-structure. Defined by the PDB web services interface.
-
-=cut
-
-sub is_obsolete {
-    my $self  = shift;
-    my $pdbid = _to_string(shift);
-    my $ret   = $self->_call(
-        'isStructureIdObsolete', $pdbid
-    );
-    return $ret;
-}
-
-=item is_unreleased ( PDBID )
-
-Checks whether or not the specified PDB ID corresponds to an unreleased
-structure. Implemented for orthogonality, all this does is check
-if C<get_status> returns C<UNRELEASED>.
-
-=cut
-
-sub is_unreleased {
-    my $self = shift;
-    return $self->get_status(@_) eq 'UNRELEASED';
-}
-
-=item is_model ( PDBID )
-
-Checks whether or not the specified PDB ID corresponds to a model
-structure. Implemented for orthogonality, all this does is check
-if C<get_status> returns C<MODEL>.
-
-=cut
-
-sub is_model {
-    my $self = shift;
-    return $self->get_status(@_) eq 'MODEL';
-}
-
-=item is_unknown ( PDBID )
-
-Checks whether or not the specified PDB ID is unknown. Implemented
-for orthogonality, all this does is check if C<get_status> returns
-C<UNKNOWN>.
-
-=cut
-
-sub is_unknown {
-    my $self = shift;
-    return $self->get_status(@_) eq 'UNKNOWN';
-}
-
-=back
-
 =head3 UNTESTED
 
 The following methods are defined by the PDB web services interface, so
@@ -747,7 +844,7 @@ they are wrapped here, but they have not been tested.
 
 =over 4
 
-=item get_annotations ( STATE_FILE )
+=item get_annotations( $STATE_FILE )
 
 Given a string in the format of a ViewState object from Protein
 Workshop, returns another ViewState object.
@@ -755,6 +852,9 @@ Workshop, returns another ViewState object.
 =cut
 
 sub get_annotations {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self       = shift;
     my $state_file = _to_string(shift);
     my $ret        = $self->_call(
@@ -763,13 +863,16 @@ sub get_annotations {
     return $ret;
 }
 
-=item get_atom_site ( PDBID )
+=item get_atom_site( $PDBID )
 
 Returns the first atom site object for a structure.
 
 =cut
 
 sub get_atom_site {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self  = shift;
     my $pdbid = _to_string(shift);
     my $ret   = $self->_call(
@@ -778,13 +881,16 @@ sub get_atom_site {
     return $ret;
 }
 
-=item get_atom_sites ( PDBID )
+=item get_atom_sites( $PDBID )
 
 Returns the atom site objects for a structure.
 
 =cut
 
 sub get_atom_sites {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self  = shift;
     my $pdbid = _to_string(shift);
     my $ret   = $self->_call(
@@ -793,13 +899,16 @@ sub get_atom_sites {
     return $ret;
 }
 
-=item get_domain_fragments ( PDBID , CHAINID , METHOD )
+=item get_domain_fragments( $PDBID , $CHAINID , $METHOD )
 
 Finds all structural protein domain fragments for a given structure.
 
 =cut
 
 sub get_domain_fragments {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self    = shift;
     my $pdbid   = _to_string(shift);
     my $chainid = _to_string(shift);
@@ -810,13 +919,16 @@ sub get_domain_fragments {
     return $ret && wantarray ? @$ret : $ret;
 }
 
-=item get_first_struct_conf ( PDBID )
+=item get_first_struct_conf( $PDBID )
 
 Finds the first struct_conf for the given structure.
 
 =cut
 
 sub get_first_struct_conf {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self  = shift;
     my $pdbid = _to_string(shift);
     my $ret   = $self->_call(
@@ -825,13 +937,16 @@ sub get_first_struct_conf {
     return $ret;
 }
 
-=item get_first_struct_sheet_range ( PDBID )
+=item get_first_struct_sheet_range( $PDBID )
 
 Finds the first struct_sheet_range for the given structure.
 
 =cut
 
 sub get_first_struct_sheet_range {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self  = shift;
     my $pdbid = _to_string(shift);
     my $ret   = $self->_call(
@@ -840,13 +955,16 @@ sub get_first_struct_sheet_range {
     return $ret;
 }
 
-=item get_struct_confs ( PDBID )
+=item get_struct_confs( $PDBID )
 
 Finds the struct_confs for the given structure.
 
 =cut
 
 sub get_struct_confs {
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
     my $self  = shift;
     my $pdbid = _to_string(shift);
     my $ret   = $self->_call(
@@ -855,44 +973,51 @@ sub get_struct_confs {
     return $ret;
 }
 
-=item get_struct_sheet_ranges ( PDBID )
+=item get_struct_sheet_ranges( $PDBID )
 
 Finds the struct_sheet_ranges for the given structure.
 
 =cut
 
 sub get_struct_sheet_ranges {
-    my $self  = shift;
-    my $pdbid = _to_string(shift);
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
+    my($self, $pdbid) = _wrap(\@_ => [SELF, STRING]);
     my $ret   = $self->_call(
         'getStructSheetRanges', $pdbid
     );
     return $ret;
 }
 
-=item get_structural_genomics_pdbids ( )
+=item get_structural_genomics_pdbids( )
 
 Finds info for structural genomics structures.
 
 =cut
 
 sub get_structural_genomics_pdbids {
-    my $self = shift;
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
+    my $self = _wrap(\@_ => [SELF]);
     my $ret  = $self->_call(
         'getStructureGenomicsPdbIds'
     );
     return $ret && wantarray ? @$ret : $ret;
 }
 
-=item xml_query ( XML )
+=item xml_query( $XML )
 
 Runs any query that can be constructed, pretty much.
 
 =cut
 
 sub xml_query {
-    my $self = shift;
-    my $xml  = _to_string(shift);
+    unshift @_, __PACKAGE__ # add the package name unless already there
+        unless defined($_[0]) && UNIVERSAL::isa($_[0], __PACKAGE__);
+
+    my($self, $xml) = _wrap(\@_ => [SELF, STRING]);
     my $ret  = $self->_call(
         'xmlQuery', $xml
     );
@@ -909,20 +1034,18 @@ sub _get_file {
     my($self, @dir) = @_;
     my $file        = pop @dir;
     my($dir, $local_path, $store, $fh);
-    if($self->{cache}) {
-        $dir        = File::Spec->catfile($self->{cache}, @dir);
+    if($self->cache) {
+        $dir        = File::Spec->catfile($self->cache, @dir);
         $local_path = File::Spec->catfile($dir, $file);
     }
-    unless($self->{cache} && ($store = new IO::File($local_path))) {
+    unless($self->cache && ($store = new IO::File($local_path))) {
         my $ftp;
-        if(
-               ($ftp = new Net::FTP($self->{ftp}, Debug => 0)) # connect
-            && $ftp->login(qw(anonymous -anonymous@))          # login
-            && $ftp->cwd(join('', map("/$_", @dir)))           # chdir
+        if(   ($ftp = new Net::FTP($self->ftp, Debug => 0)) # connect
+            && $ftp->login(qw(anonymous -anonymous@))       # login
+            && $ftp->cwd(join('', map("/$_", @dir)))        # chdir
         ) {
             # store in temporary file unless there's a cache
-            $store = IO::File->new_tmpfile unless
-                   $self->{cache}                              # cache exists
+            $store = IO::File->new_tmpfile unless $self->cache # cache exists
                 && File::Path::mkpath($dir)                    # mkdir
                 && ($store = new IO::File($local_path, '+>')); # create file
             
@@ -932,7 +1055,7 @@ sub _get_file {
             }
             else {
                 undef $store;
-                $self->{cache} and unlink $local_path;
+                $self->cache and unlink $local_path;
             }
             
             # clean up
@@ -953,11 +1076,31 @@ sub _get_file {
 
 sub _call {
     my $self   = shift;
-    my $result = $self->service->call(@_);
+    my $result = $self->soap->call(@_);
     confess $result->faultstring if $result->fault;
     return $result->result;
 }
 
+sub _wrap {
+    my @data = @{shift()};
+    my @type = @{shift()};  
+    return map {
+        my $type = shift @type;
+        if($type == BOOLEAN) {
+            $_ = SOAP::Data->type(boolean => ($_ ? 1 : 0));
+        }
+        elsif($type == DOUBLE) {
+            $_ = SOAP::Data->type(double => $_);
+        }
+        elsif($type == INT) {
+            $_ = SOAP::Data->type('int' => $_);
+        }
+        elsif($type == STRING) {
+            $_ = SOAP::Data->type(string => $_);
+        }
+    $_} @data;
+}
+
 sub _to_int {
     my $var = shift;
     return SOAP::Data->type('int' => $var);
@@ -1037,13 +1180,14 @@ M. (1977). I<Eur. J. Biochem.> B<80>(2), 319-324.
 
 =head1 SEE ALSO
 
-The PDB can be accessed via the web at E<lt>http://www.pdb.org/E<gt>. The
+The PDB can be accessed via the web at L<http://www.pdb.org/>. The
 Java API documentation for the PDB's web services is located at
-E<lt>http://www.rcsb.org/robohelp_f/webservices/pdbwebservice.htmlE<gt>.
+L<http://www.rcsb.org/robohelp_f/webservices/pdbwebservice.html>.
 
 =head1 BUGS
 
-Please report them.
+Please report them:
+L<http://rt.cpan.org/Public/Dist/Display.html?Name=WWW-PDB>
 
 =head1 AUTHOR
 
